---------------------------------------------------------------------------------
-- Navision Relationship between contacts and companies
---------------------------------------------------------------------------------


-- Adding the NAV "Contact Business Relationship" to acs_rels
drop function nav_import_contact_business_relationships();
create or replace function nav_import_contact_business_relationships ()
returns integer as $BODY$
DECLARE
        row			RECORD;
	v_user_id		integer;
	v_company_id		integer;
	v_exists_p		integer;
	v_group_id		integer;
BEGIN
    FOR row IN
        select	*
	from	nav."Contact Business Relation"
	order by "Contact No_", "No_"
    LOOP
	--------------------------------------------------------------------
	-- Lookup user
	select min(person_id) into v_user_id from persons
	where nav_contact_nr ~ row."Contact No_";
	IF v_user_id is null THEN
		RAISE WARNING 'Did not find user=%', row."Contact No_";
		continue;
	END IF;

	--------------------------------------------------------------------
	-- Lookup company
	select company_id into v_company_id from im_companies
	where nav_vendor_nr = row."No_";
	IF v_company_id is null THEN
		select company_id into v_company_id from im_companies
		where nav_customer_nr = row."No_";
	END IF;
	IF v_company_id is null THEN
		RAISE WARNING 'Did not find company=%', row."No_";
		continue;
	END IF;

	--------------------------------------------------------------------
	-- Create the relationship between company and contact
	RAISE NOTICE 'Making % member of %', im_name_from_user_id(v_user_id), v_company_id;
	PERFORM im_biz_object_member__new (
		null,
		'im_biz_object_member',
		v_company_id,			-- the business object
		v_user_id,			-- the person related to biz_object
		1300,				-- role: Full Member
		null,				-- percentage
		null,				-- creation user
		'0.0.0.0'			-- creation IP
	);

	--------------------------------------------------------------------
	-- Make the contact the "primary_contact_id" of the company
	-- and set the nav_vendor_nr on the person
	update im_companies
	set primary_contact_id = v_user_id
	where company_id = v_company_id;

	update persons
	set nav_primary_contact_for_company_id = v_company_id
	where person_id = v_user_id;

	--------------------------------------------------------------------
	-- Make the user member of the groups Providers or Customers
	IF 'Z' = substring(row."Business Relation Code" for 1) THEN
		RAISE NOTICE 'User is a Customer: %', row."Contact No_";
		select group_id into v_group_id from groups 
		where group_name = 'Customers';
	ELSE
		RAISE NOTICE 'User is a Provider: %', row."Contact No_";
		select group_id into v_group_id from groups 
		where group_name = 'Freelancers';
	END IF;

	-- Check if the user is already a member of that group
	select	count(*) 
	into	v_exists_p 
	from	acs_rels r
	where	rel_type = 'membership_rel' and
		object_id_one = v_group_id and
		object_id_two = v_user_id;

	-- Add the user to the group if he is not a member already.
	IF v_exists_p = 0 THEN
		RAISE NOTICE 'Adding User to Group: %', row."Contact No_";
		PERFORM membership_rel__new(
			null,			-- rel_id, default null  
			'membership_rel',	-- object_type, default membership_rel
			v_group_id,		-- object_id_one
			v_user_id,		-- object_id_two
			'approved',		-- new__member_state, default approved
			null,			-- creation_user, default null
			'0.0.0.0'		-- creation_ip, default null
		);
	END IF;


    END LOOP;
    RETURN 0;
END;$BODY$ language 'plpgsql';
select nav_import_contact_business_relationships ();



