---------------------------------------------------------------------------------
-- Navision Contacts
---------------------------------------------------------------------------------


-- convert empty strings in NULL values to allow coalesce() to work
--
ALTER TABLE nav."Contact" ALTER COLUMN "First Name" DROP NOT NULL;
update nav."Contact" set "First Name" = null where "First Name" = '';
--
ALTER TABLE nav."Contact" ALTER COLUMN "Surname" DROP NOT NULL;
update nav."Contact" set "Surname" = null where "Surname" = '';


-- Actually create new users by going through the nav."Contact" table
-- line by line and inserting the user into the DB.
--
drop function nav_import_contact();
create or replace function nav_import_contact ()
returns integer as $BODY$
DECLARE
        row			RECORD;
	v_user_id		integer;
	v_string		varchar;
	v_exists_p		integer;
	v_username		varchar;
	v_group_id		integer;
	v_email			varchar;
	v_email_domain		varchar;
	v_email_name		varchar;
	v_customer_group_id	integer;
	v_first_names		varchar;
	v_last_name		varchar;
BEGIN
    FOR row IN
        select	*,
		coalesce("First Name", substring("Name" from '^([^ ]+)')) as first_names,
		coalesce("Surname", substring("Name" from '^[^ ]+ (.*)$')) as last_name
	from	nav."Contact"
	order by "No_"
    LOOP
	v_email := substring(lower(trim(row."E-Mail")) from '[a-zA-Z0-9_\_\.]+\@[a-zA-Z0-9_\_\.]+');
	v_email_domain := substring(v_email from '@(.*)');
	v_email_name := substring(v_email from '(.*)@');
	v_email := v_email_name || '@' || v_email_domain;

	-- Transform first and last name
	v_first_names := row.first_names;
	v_last_name := row.last_name;

	-- Accept empty last names for company contacts. Replace by first name
	IF row."Type" = 0 and (v_last_name is null OR v_last_name = '') THEN
		v_last_name := v_first_names;
	END IF;

	-- Accept empty email for company contacts.
	IF row."Type" = 0 and (v_email is null OR v_email = '') THEN
		v_email = v_first_names ||'.'||v_last_name||'@nowhere.com';
	END IF;

	RAISE NOTICE 'Processing User: first=%, last=%, email=%, no=%, company_no=%', 
		v_first_names, v_last_name, v_email, row."No_", row."Company No_";

	IF v_first_names is null OR v_first_names = '' THEN 
	   	RAISE NOTICE 'Skipping User (empty first_names): % % (%)', v_first_names, v_last_name, v_email;
		continue; 
	END IF;
	IF v_last_name is null OR v_last_name = '' THEN 
	   	RAISE NOTICE 'Skipping User (empty last_name): % % (%)', v_first_names, v_last_name, v_email;
		continue; 
	END IF;
	IF v_email is null OR v_email = '' THEN
	   	RAISE NOTICE 'Skipping User (empty email): % % (%)', v_first_names, v_last_name, v_email;
		continue; 
	END IF;

	-- Check if the user already exists / has been imported already
	-- during the last run of this script. PrimKey is first_names+last_name.
	-- Check 1. ContactNo_, 2. Email and 3. Name
	select min(person_id) into v_user_id from persons
	where nav_contact_nr ~ row."No_";

	IF v_user_id is null  THEN
		select	min(party_id) into v_user_id from parties p
		where	trim(lower(p.email)) = trim(lower(v_email));
	END IF;
	IF v_user_id is null THEN
		select	min(person_id) into v_user_id from persons p
		where	(trim(lower(p.first_names)) = trim(lower(v_first_names)) and
			trim(lower(p.last_name)) = trim(lower(v_last_name)));
	END IF;

	-- Create a new user if the user wasnt there
	IF v_user_id is null THEN
	   	RAISE NOTICE 'Insert User: % % (%)', v_first_names, v_last_name, v_email;

		-- Create a default username
		v_username := v_email;

		v_user_id := acs__add_user(
			null, 'user', now(), 0, '0.0.0.0', 
			null, v_username, v_email, null,
			v_first_names, v_last_name,
			'hashed_password', 'salt', 
			v_username, 't', 'approved'
		);
		INSERT INTO users_contact (user_id) VALUES (v_user_id);
		INSERT INTO im_employees (employee_id) VALUES (v_user_id);
		INSERT INTO im_freelancers (user_id) VALUES (v_user_id);

	ELSE
		SELECT username into v_username from users
		where user_id = v_user_id;
	END IF;


	-----------------------------------------------------------------
	-- This is the main part of the import process, this part is 
	-- executed for every line in the import_users table every time
	-- this script is executed.
	-- Update the users information, no matter whether its a new or
	-- an already existing user.

	RAISE NOTICE 'Update User: email=%, id=%', v_email, v_user_id;
	update users_contact set
			wa_line1 = row."Address",
			wa_line2 = row."Address 2",
			wa_city = row."City",
			wa_postal_code = row."Post Code",
			work_phone = row."Phone No_",
			fax = row."Telex No_",
			pager = row."Pager"
	where user_id = v_user_id;

	update users set
			username = v_username
	where user_id = v_user_id;

	update persons set
			first_names = v_first_names,
			last_name = v_last_name,
			nav_middle_name = row."Middle Name",
			nav_skype = row."Skype"
	where person_id = v_user_id;

	update parties set
			url = row."Home Page"
	where party_id = v_user_id;

	update im_employees set
			job_title = row."Job Title"
	where employee_id = v_user_id;


	----------------------------------------------------------------
	-- Update persons.nav_contact_nr. It's a space separated list of NAV contact_nrs
	select	coalesce(nav_contact_nr, '') into v_string
	from	persons where person_id = v_user_id;
	
	IF v_string !~ row."No_" THEN
		IF v_string is null or v_string = ''
		THEN v_string := row."No_";
		ELSE v_string := v_string || ' ' || row."No_";
		END IF;

		update persons
		set nav_contact_nr = v_string
		where person_id = v_user_id;
	END IF;

	----------------------------------------------------------------
	-- Make the new user a member of group Customers or Providers

	-- select out the ID of the Customers group
	IF 'Z' = substring(row."No_" for 1) THEN
		RAISE NOTICE 'User is a Customer: %', v_email;
		select group_id into v_customer_group_id from groups 
		where group_name = 'Customers';
	ELSE
		RAISE NOTICE 'User is a Provider: %', v_email;
		select group_id into v_customer_group_id from groups 
		where group_name = 'Freelancers';
	END IF;

	-- Check if the user is already a member of that group
	select	count(*) 
	into	v_exists_p 
	from	acs_rels r
	where	rel_type = 'membership_rel' and
		object_id_one = v_customer_group_id and
		object_id_two = v_user_id;

	-- Add the user to the group if he is not a member already.
	IF false AND v_exists_p = 0 THEN
		RAISE NOTICE 'Adding User to Group: %', v_email;
		PERFORM membership_rel__new(
			null,			-- rel_id, default null  
			'membership_rel',	-- object_type, default membership_rel
			v_customer_group_id,	-- object_id_one
			v_user_id,		-- object_id_two
			'approved',		-- new__member_state, default approved
			null,			-- creation_user, default null
			'0.0.0.0'		-- creation_ip, default null
		);
	END IF;

    END LOOP;
    RETURN 0;
END;$BODY$ language 'plpgsql';
-- select nav_import_contact ();




-- Actually create new users by going through the nav."Contact" table
-- line by line and inserting the user into the DB.
--
drop function nav_import_contact_memberships();
create or replace function nav_import_contact_memberships ()
returns integer as $BODY$
DECLARE
        row			RECORD;
	v_user_id		integer;
	v_company_id		integer;
BEGIN
    FOR row IN
        select	*,
		coalesce("First Name", substring("Name" from '^([^ ]+)')) as first_names,
		coalesce("Surname", substring("Name" from '^[^ ]+ (.*)$')) as last_name
	from	nav."Contact"
	where	" Company No_" = 'ZL00140'
	order by "No_"
    LOOP
	RAISE NOTICE 'Contact Memberships: Processing User: name=%, email=%, no=%, company_no=%', 
	row."Name", row."E-Mail", row."No_", row."Company No_";
	
	--------------------------------------------------------------------
	-- Interpret the "Company No_".
	-- This key references a "Contact" of type 0=Company, which in turn
	-- is connected to the "Vendor" via "Contact Business Relation".

	v_user_id := (select min(person_id) from persons where nav_contact_nr ~ row."No_");
	IF v_user_id is null THEN
		RAISE WARNING 'Contact Memberships: Did not find user no=%, name=%', row."No_", row."Name";
		continue;
	END IF;

	-- Check for the type of contact:
	-- 	0=Company
	--	1=User
	IF 0 = row."Type" THEN

		RAISE NOTICE 'Contact Memberships: Found a "company": no=%, name=%', row."No_", row."Name";
		-- Type=0: A company
		select	min(company_id) into v_company_id
		from	im_companies
		where	(nav_customer_nr in (select "No_" from nav."Contact Business Relation" where "Contact No_" = row."No_") OR
			nav_vendor_nr in (select "No_" from nav."Contact Business Relation" where "Contact No_" = row."No_"))
		;
	ELSE
		RAISE NOTICE 'Contact Memberships: Found a "person": no=%, name=%', row."No_", row."Name";
		-- Type=1: A user that represents a company
		v_company_id := (select min(company_id) from im_companies where nav_vendor_nr in (
			select "No_" from nav."Vendor" where "No_" in (
				select "No_" from nav."Contact Business Relation" where "Contact No_" in (
					select "No_" from nav."Contact" where "No_" = row."Company No_"
				)
			)
		));
	END IF;

	IF v_company_id is null THEN
		RAISE WARNING 'Contact Memberships: Did not find company no=%', row."Company No_";
		continue;
	END IF;

	RAISE NOTICE 'Contact Memberships: Adding User % to Company % for %', row."E-Mail", v_company_id, row."Company No_";
	PERFORM im_biz_object_member__new (
		null,
		'im_biz_object_member',
		v_company_id,			-- the business object
		v_user_id,			-- the person related to biz_object
		1300,				-- role: Full Member
		null,				-- percentage
		null,				-- creation user
		'0.0.0.0'			-- creation IP
	);

    END LOOP;
    RETURN 0;
END;$BODY$ language 'plpgsql';
select nav_import_contact_memberships ();

