--------------------------------------------------------------------------------
-- Check for users that are not included in Secure Access anymore
---------------------------------------------------------------------------------


-- Find out which users in po are not in Secure Access 
--
drop function nav_prune_users();
create or replace function nav_prune_users ()
returns integer as $BODY$
DECLARE
        row_user		RECORD;
        row_group		RECORD;
	v_count			integer;
	v_user_id		integer;
BEGIN
    v_count := 0;
    FOR row_user IN
	select	*
	from	(select	u.*,
			auth.short_name as authority,
			auth.short_name || '\\' || u.username as windows_login
		from	cc_users u,
			auth_authorities auth
		where	u.authority_id = auth.authority_id and
			u.member_state != 'banned'
		) u
	where
		windows_login not in (
		        select	lower(u."Loginname")
			from	sa."AR_User" u
		)
		and authority != 'local'
	order by u.user_id
    LOOP
	v_count := v_count + 1;

	RAISE NOTICE 'Disabling User #%: First=%, Last=%, Uname=%, Login=% => %', 
		     v_count, row_user.first_names, row_user.last_name, row_user.username, row_user.windows_login, row_user.user_id;

	update membership_rels
	set member_state = 'banned'
	where rel_id in (
		select	rel_id
		from	acs_rels
		where	object_id_one = -2 and
			object_id_two = row_user.user_id
		);

    END LOOP;
    RETURN 0;
END;$BODY$ language 'plpgsql';
select nav_prune_users ();



--------------------------------------------------------------------------------
-- Prune user grous
---------------------------------------------------------------------------------


-- Find out which users in po are not in Secure Access 
--
drop function nav_prune_user_groups();
create or replace function nav_prune_user_groups ()
returns integer as $BODY$
DECLARE
        row_user		RECORD;
        row_group		RECORD;

	v_count			integer;
	v_user_id		integer;
BEGIN
    v_count := 0;
    FOR row_user IN
	select	*
	from
		(select	u.*,
			auth.short_name || '\\' || u.username as windows_login
		from	cc_users u,
			auth_authorities auth
		where	u.authority_id = auth.authority_id
		) u
	where
		windows_login not in (
		        select	lower(u."Loginname")
			from	sa."AR_User" u
		)
	order by u.user_id
    LOOP
	v_count := v_count + 1;

	RAISE NOTICE 'Processing User #%: First=%, Last=%, Login=%, Email=%', v_count, row_user.first_names, row_user.last_name, row_user.windows_login, row_user.email;

	-- Delete the user from groups he is not a member of anymore
	FOR row_group IN
		select	g.group_name,
			r.rel_id
		from	acs_rels r, groups g, membership_rels mr
		where	r.rel_id = mr.rel_id and
			r.object_id_one = g.group_id and
			mr.member_state = 'approved' and
			r.object_id_two = v_user_id and
			g.group_name not in (
				select	sgm.po_group_name
				from	sa.view_ar_relationshipall_parent_valid var,
					sa_group_mapping sgm
				where	var."ObjectName" = sgm.sa_group_name and
					var."ChildObjectID" in (
						-- build the union of all group memberships of all accounts
						select	"ObjectID"
						from	sa."AR_User"
						where	lower(trim("EmailAddress")) = v_email
					) 
			) and
			g.group_name not in ('Registered Users', 'The Public')
	LOOP
		RAISE NOTICE 'Deleting user % from group %', v_loginname, row_group.group_name;

		-- Get the PO group_id
		select group_id into v_group_id from groups where group_name = row_group.group_name;

		RAISE NOTICE 'Deleting: user=%, group=%', v_user_id, v_group_id;
		PERFORM membership_rel__delete(row_group.rel_id);
	END LOOP;

    END LOOP;
    RETURN 0;
END;$BODY$ language 'plpgsql';
-- select nav_prune_user_groups ();

